@@ -99,6 +99,14 @@ class TTree(object):
         self._tree.write(context, cursor, name, key, copy(keycursor), util)
 
     def extend(self, branchdict, flush=True):
+        #Convert to numpy arrays of required dtype
+        for key, value in branchdict.items():
+            branchdict[key] = numpy.array(value, dtype=self._branches[key]._branch.type, copy=False)
+
+            #Temporary place holder
+            if branchdict[key].ndim != 1:
+                raise NotImplementedError("Multi dimensional array support is coming soon")
+
         #Check for equal number of values in baskets
         values = iter(branchdict.values())
         first = next(values)
@@ -267,6 +275,9 @@ class TBranch(object):
 
     _tree_size = struct.Struct(">qqq")
     def newbasket(self, items):
+        if len(items) == 0:
+            return
+
         self._branch.fields["_fWriteBasket"] += 1
 
         if self._branch.fields["_fWriteBasket"] >= self._branch.fields["_fMaxBaskets"]:
@@ -332,6 +343,10 @@ class TBranch(object):
         self._branch.fields["_fBasketEntry"][self._branch.fields["_fWriteBasket"]] = self._branch.fields["_fEntries"]
         self._branch.fields["_fEntryNumber"] += len(items)
         basketdata = numpy.array(items, dtype=self._branch.type, copy=False)
+
+        if basketdata.ndim != 1:
+            raise NotImplementedError("Multi dimensional array support is coming soon")
+
         givenbytes = basketdata.tostring()
         cursor = uproot.write.sink.cursor.Cursor(self._branch.file._fSeekFree)
         self._branch.fields["_fBasketSeek"][self._branch.fields["_fWriteBasket"] - 1] = cursor.index
@@ -636,8 +651,8 @@ class TTreeImpl(object):
             values = [values]
         low = 0
         buff += cursor.put_string(b"") + cursor.put_fields(self._format_tobjarray1, size, low)
-        for value in values:
-            buff += self.util.put_objany(cursor, (value, classname), self.write_keycursor)
+        for i in range(len(self.fields["_fLeaves"])):
+            buff += self.util.put_objany(cursor, (values[i], classname[i] if type(classname)==list else classname), self.write_keycursor)
         length = len(buff) + self._format_cntvers.size
         cnt = numpy.int64(length - 4) | uproot.const.kByteCountMask
         return copy_cursor.put_fields(self._format_cntvers, cnt, vers) + buff
@@ -712,7 +727,8 @@ class TTreeImpl(object):
         if self.fields["_fBranches"] == []:
             buff += self.put_tobjarray(cursor, self.fields["_fLeaves"], "TLeaf")
         else:
-            buff += self.put_tobjarray(cursor, self.fields["_fLeaves"][0][0], classname=self.fields["_fLeaves"][0][1])
+            buff += self.put_tobjarray(cursor, [self.fields["_fLeaves"][i][0] for i in range(len(self.fields["_fLeaves"]))],
+                                       classname=[self.fields["_fLeaves"][i][1] for i in range(len(self.fields["_fLeaves"]))])
         buff += (self.util.put_objany(cursor, (self.fields["_fAliases"], "TList"), self.write_keycursor) +
                  self.put_tarray(cursor, self.fields["_fIndexValues"]) +
                  self.put_tarray(cursor, self.fields["_fIndex"]) +
